@@ -95,7 +95,7 @@ try:
 except:
 	has_numpy = False
 
-__version__ = '1.0.1'
+__version__ = '1.0.2'
 
 def NoneStr(obj): return 'NULL'
 
@@ -180,9 +180,9 @@ def OtherStr(obj):
 			return SeqStr(list(obj))
 	return repr(obj)
 
-base_tps = [NoneType, bool, int, long, float, complex, str, unicode, list, tuple, set, frozenset, dict]
+base_tps = [type(None), bool, int, long, float, complex, str, unicode, list, tuple, set, frozenset, dict] # use type(None) instead of NoneType since the latter cannot be found in the types module in Python 3
 base_tps.reverse()
-str_func = {NoneType:NoneStr, bool:BoolStr, int:repr, long:LongStr, float:repr, complex:ComplexStr, str:repr, unicode:repr, list:SeqStr, tuple:SeqStr, set:SeqStr, frozenset:SeqStr, dict:DictStr}
+str_func = {type(None):NoneStr, bool:BoolStr, int:repr, long:LongStr, float:repr, complex:ComplexStr, str:repr, unicode:repr, list:SeqStr, tuple:SeqStr, set:SeqStr, frozenset:SeqStr, dict:DictStr}
 
 def Str4R(obj):  
 	'''
@@ -206,7 +206,7 @@ class R: # (object):
 	'''
 	A Python class to enclose an R process.
 	'''
-	__Rfun = '''.getRvalue4Python__ <- function(x) {
+	__Rfun = '''.getRvalue4Python__ <- function(x, use_dict=FALSE) {
 	has_numpy <- %s
 	if (has_numpy) {
 		headstr <- 'numpy.array('
@@ -214,15 +214,19 @@ class R: # (object):
 	else headstr <- tailstr <- ''
 	NullStr <- function(x) 'None'
 	VectorStr <- function(x) {
+		#nms <- names(x)
+		#if (!is.null(nms) &&  length(nms)>0) return(ListStr(as.list(x)))
 		complx <- is.complex(x)
 		if (is.character(x)) x <- paste('"', x, '"', sep='')
-		if (length(x)==1) x <- paste(x)
+		else if (is.logical(x)) x <- ifelse(x, 'True', 'False')
+		if (length(x)==1) x <- paste(x) # convert to character, or use "gettext", "as.character"
 		else x <- paste(headstr, '[', paste(x, collapse=', '), ']', tailstr, sep='')
 		if (complx) x <- gsub('i', 'j', x)
 		return(x) }
 	MatrixStr <- function(x) {
 		complx <- is.complex(x)
 		if (is.character(x)) x <- matrix(paste('"', x, '"', sep=''), nrow=nrow(x))
+		else if (is.logical(x)) x <- ifelse(x, 'True', 'False')
 		x <- apply(x, 1, function(r) paste('[', paste(r, collapse=', '), ']', sep=''))
 		x <- paste(headstr, '[', paste(x, collapse=', '), ']', tailstr, sep='')
 		if (complx) x <- gsub('i', 'j', x)
@@ -234,6 +238,7 @@ class R: # (object):
 		if (ndim == 2) return(MatrixStr(x))
 		# ndim >= 3
 		if (is.character(x)) x <- array(paste('"', x, '"', sep=''), dim=dim(x))
+		else if (is.logical(x)) x <- ifelse(x, 'True', 'False')
 		for (i in seq(ndim-1)) 
 			x <- apply(x, seq(dim(x))[-1], function(r) paste('[', paste(r, collapse=', '), ']', sep=''))
 		x <- paste(headstr, '[', paste(x, collapse=', '), ']', tailstr, sep='')
@@ -270,10 +275,14 @@ class R: # (object):
 	ListStr <- function(x) {
 		nms <- names(x) # get column names
 		x <- sapply(x, Str4Py)
-		if (!is.null(nms)) {
+		if (!is.null(nms) &&  length(nms)>0) {
 			nms <- paste('"', nms, '"', sep='')
-			x <- sapply(seq(nms), function(i) paste('(', nms[i], ',', x[i], ')') ) }
-		x <- paste('[', paste(x, collapse=', '), ']', sep='')
+			x <- sapply(seq(nms), function(i) paste('(', nms[i], ',', x[i], ')') ) 
+			if (use_dict) x <- paste('dict([', paste(x, collapse=', '), '])', sep='')
+			else x <- paste('[', paste(x, collapse=', '), ']', sep='')
+			}
+		else
+			x <- paste('[', paste(x, collapse=', '), ']', sep='')
 		return(x) }
 	Str4Py <- function(x) {
 		# no considering on NA, Inf, ...
@@ -284,6 +293,7 @@ class R: # (object):
 		if (is.matrix(x) || is.array(x)) return(ArrayStr(x))
 		if (is.data.frame(x)) return(DataFrameStr(x))
 		if (is.list(x)) return(ListStr(x))
+		return(Str4Py(as.character(x))) # other objects will be convert to character, or use "gettext"
 		return(NullStr(x)) }
 	Str4Py(x)
 	}
@@ -303,7 +313,7 @@ class R: # (object):
 	''' 
 	_DEBUG_MODE = True
 
-	def __init__(self, RCMD='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, host='localhost', user=None, ssh='ssh', return_err=True):
+	def __init__(self, RCMD='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=False, host='localhost', user=None, ssh='ssh', return_err=True):
 		'''
 		RCMD: The name of a R interpreter, path information should be included
 			if it is not in the system search path.
@@ -315,6 +325,8 @@ class R: # (object):
 			longer than this value.
 		use_numpy: Used as a boolean value. A False value will disable numpy
 			even if it has been imported.
+		use_dict: named list will be returned a dict if use_dict is True,
+			otherwise it will be a list of tuples (name, value)
 		host: The computer name (or IP) on which the R interpreter is
 			installed. The value "localhost" means that R locates on the the
 			localhost computer. On POSIX systems (including Cygwin environment
@@ -339,11 +351,12 @@ class R: # (object):
 		# underscore (DunderAlias) at both ends of the name) may look at the
 		# class rather than the instance even for old-style classes."
 		self.__dict__.update({
-			'wait0':wait0, 
-			'wait':wait, 
-			'max_len':max_len, 
-			'localhost':host=='localhost', 
-			'tail':sys.platform=='win32' and '\r\n' or '\n'})
+			'wait0' : wait0, 
+			'wait' : wait, 
+			'max_len' : max_len, 
+			'use_dict' : use_dict,
+			'localhost' : host=='localhost', 
+			'tail' : sys.platform=='win32' and '\r\n' or '\n'})
 
 		RCMD = shlex.split(RCMD) #re.split(r'\s', RCMD)
 		if not self.localhost:
@@ -361,9 +374,9 @@ class R: # (object):
 			info.wShowWindow = subprocess.SW_HIDE
 		else: info = None
 		self.__dict__.update({
-			'prog':Popen(RCMD, stdin=PIPE, stdout=PIPE, stderr=return_err and subprocess.STDOUT or None, startupinfo=info), 
-			'has_numpy':use_numpy and has_numpy, 
-			'Rfun':self.__class__.__Rfun % ((use_numpy and has_numpy) and 'TRUE' or 'FALSE')})
+			'prog' : Popen(RCMD, stdin=PIPE, stdout=PIPE, stderr=return_err and subprocess.STDOUT or None, startupinfo=info), 
+			'has_numpy' : use_numpy and has_numpy, 
+			'Rfun' : self.__class__.__Rfun % ((use_numpy and has_numpy) and 'TRUE' or 'FALSE')})
 		self.__call__(self.Rfun)
 		to_discard = recv_some(self.prog, e=0, t=wait0)
 
@@ -412,7 +425,7 @@ class R: # (object):
 		if len(rlt) == 1: rlt = rlt[0]
 		return rlt
 
-	def __getitem__(self, obj, use_try=None):
+	def __getitem__(self, obj, use_try=None, use_dict=None): # to model r['XXX']
 		'''
 		Get the value of an R variable or expression. The return value is
 		converted to the corresponding Python object.
@@ -420,11 +433,14 @@ class R: # (object):
 		obj: a string - the name of an R variable, or an R expression
 		use_try: use "try" function to wrap the R expression. This can avoid R
 			crashing if the obj does not exist in R.
+		use_dict: named list will be returned a dict if use_dict is True,
+			otherwise it will be a list of tuples (name, value)
 		'''
 		if obj.startswith('_'):
 			raise RError('Leading underscore ("_") is not permitted in R variable names!')
 		use_try = use_try or self._DEBUG_MODE
-		cmd = '.getRvalue4Python__(%s)' % obj
+		if use_dict is None: use_dict = self.use_dict
+		cmd = '.getRvalue4Python__(%s, use_dict=%s)' % (obj, use_dict and 'TRUE' or 'FALSE')
 		rlt = self.__call__(cmd, use_try=use_try)
 		head = len((use_try and 'try({%s})%s[1] ' or '%s%s[1] ') % (cmd, self.tail)) 
 		tail = len(rlt) - len(self.tail) # - len('"')
@@ -434,7 +450,7 @@ class R: # (object):
 			raise RError(rlt)
 		return rlt
 
-	def __setitem__(self, obj, val):
+	def __setitem__(self, obj, val): # to model r['XXX']
 		'''
 		Assign a value (val) to an R variable (obj).
 
@@ -454,17 +470,23 @@ class R: # (object):
 		send_all(self.prog, 'q("no")'+self.tail)
 		self.prog = None
 
-	def __getattr__(self, obj):
+	def __getattr__(self, obj, use_dict=None): # to model r.XXX
 		'''
 		obj: a string - the name of an R variable
+		use_dict: named list will be returned a dict if use_dict is True,
+			otherwise it will be a list of tuples (name, value)
 		'''
+		# Overriding __getattr__ is safer than __getattribute__ since it is
+		# only called as a last resort i.e. if there are no attributes in the
+		# instance that match the name
 		try:
-			rlt = self.__getitem__(obj)
+			if use_dict is None: use_dict = self.use_dict
+			rlt = self.__getitem__(obj, use_dict=use_dict)
 		except:
 			raise RError('No this object!')
 		return rlt
 
-	def __setattr__(self, obj, val):
+	def __setattr__(self, obj, val): # to model r.XXX
 		if obj in self.__dict__ or obj in self.__class__.__dict__: # or obj.startswith('_'):
 			self.__dict__[obj] = val # for old-style class
 			#object.__setattr__(self, obj, val) # for new-style class
@@ -477,13 +499,16 @@ class R: # (object):
 		else:
 			self.__delitem__(obj)
 
-	def get(self, obj, default=None):
+	def get(self, obj, default=None, use_dict=None):
 		'''
 		obj: a string - the name of an R variable, or an R expression
 		default: a python object - the value to be returned if failed to get data from R
+		use_dict: named list will be returned a dict if use_dict is True,
+			otherwise it will be a list of tuples (name, value). If use_dict is
+			None, the value of self.use_dict will be used instead.
 		'''
 		try:
-			rlt = self.__getitem__(obj, use_try=True)
+			rlt = self.__getitem__(obj, use_try=True, use_dict=use_dict)
 		except:
 			if True: #val is not None:
 				rlt = default
@@ -495,7 +520,7 @@ class R: # (object):
 
 
 # for a single-round duty:
-def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, host='localhost', user=None, ssh='ssh'):
+def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=False, host='localhost', user=None, ssh='ssh'):
 	'''
 	Run a (list of) R command(s), and return the output from the STDOUT.
 
@@ -509,6 +534,8 @@ def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, host=
 		than this value.
 	use_numpy: Used as a boolean value. A False value will disable numpy even
 		if it has been imported.
+	use_dict: named list will be returned a dict if use_dict is True, otherwise
+		it will be a list of tuples (name, value).
 	host: The computer name (or IP) on which the R interpreter is
 		installed. The value "localhost" means that the R locates on the
 		the localhost computer. On POSIX systems (including Cygwin
@@ -523,7 +550,7 @@ def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, host=
 	ssh: The program to login to remote computer.
 	'''
 	if isinstance(Robj, basestring):
-		Robj = R(RCMD=Robj, wait0=wait0, wait=wait, max_len=max_len, host=host, user=user, ssh=ssh)
+		Robj = R(RCMD=Robj, wait0=wait0, wait=wait, max_len=max_len, use_numpy=use_numpy, use_dict=use_dict, host=host, user=user, ssh=ssh)
 	rlt = Robj.run(CMDS=CMDS)
 	if len(rlt) == 1: rlt = rlt[0]
 	return rlt
